﻿using Jst.HumanResources.Oa.Entities;
using Kivii;
using Kivii.HumanResources.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Jst.HumanResources.Oa.Jobs
{
    public class OaWorkToDoJob : IJob
    {
        public string Name => "OA请假同步任务";

        public string Description => "获取OA请假同步至考勤任务";

        public IJobContext TaskContext { get; set; }

        public void Dispose()
        {
        }

        public bool Execution()
        {
            return OaSync();
        }

        private bool OaSync()
        {
            var connHr = KiviiContext.GetOpenedDbConnection<Attendance>();

            var queryNotSigned = connHr.From<Attendance>();
            //只查三个月以内的
            queryNotSigned.Where(o => o.TimeResult != "Normal" & o.TimeResult != "Leave" & o.BaseCheckTime >= DateTime.Now.AddMonths(-3));
            //查出未打卡的人清单
            var notSigneds = connHr.Select(queryNotSigned);

            if (notSigneds.IsNullOrEmpty())
            {
                TaskContext.Message = $"暂无请假申请同步!";
                return true;
            }

            int total = notSigneds.Count, success = 0;

            var names = notSigneds.ConvertAll(o => o.UserName).Distinct().ToList();

            var connOa = KiviiContext.GetOpenedDbConnection<OaWorkToDo>();
            var queryOa = connOa.From<OaWorkToDo>();
            queryOa.Where(o => o.StateNow == "正常结束" & Sql.In(o.UserName, names) & o.WorkName.Contains("请假"));

            var oaWorkToDos = connOa.Select(queryOa);
            foreach (var oa in oaWorkToDos)
            {
                var beginStr = "alt=开始时间 value=\"";
                var endStr = "alt=结束时间 value=\"";
                var reasonBeginStr = "请假理由 size=31 value=";
                var reasonEndStr = "name=Text77021621";
                
                int beginReasonIndex = oa.FormContent.IndexOf(reasonBeginStr);
                int endReasonIndex = oa.FormContent.IndexOf(reasonEndStr);
                int beginIndex = oa.FormContent.IndexOf(beginStr);
                int endIndex = oa.FormContent.IndexOf(endStr);
                if (beginIndex == -1) continue;
                if (endIndex == -1) continue;
                if (!DateTime.TryParse(oa.FormContent.Substring(beginIndex + beginStr.Length, 10), out var beginTime)) continue;
                if (!DateTime.TryParse(oa.FormContent.Substring(endIndex + endStr.Length, 10), out var endtTime)) continue;

                var reason = string.Empty;
                if (beginReasonIndex != -1)
                {
                    if (endReasonIndex == -1) endReasonIndex = beginReasonIndex + reasonBeginStr.Length - 6;
                    try
                    {
                        reason = oa.FormContent.Substring(beginReasonIndex + reasonBeginStr.Length, endReasonIndex - beginReasonIndex - reasonBeginStr.Length);
                    }
                    catch { }
                }
                oa.IsSuccess = true;
                oa.TimeStr = beginTime;
                oa.LateTime = endtTime.AddDays(1);
                oa.Reason = reason;
            }
            foreach (var notSigned in notSigneds)
            {
                var oas = oaWorkToDos.Where(o => o.UserName == notSigned.UserName).ToList();
                if (oas.IsNullOrEmpty()) continue;
                var oa = oas.FirstOrDefault(o => o.IsSuccess == true & o.TimeStr >= notSigned.BaseCheckTime & notSigned.BaseCheckTime < o.LateTime);
                if (oa == null) continue;
                notSigned.TimeResult = "Leave";
                notSigned.AddOnlyProperties(o => o.TimeResult);
                notSigned.SourceType = "OA";
                notSigned.AddOnlyProperties(o => o.SourceType);
                notSigned.Summary = $"{oa.Reason}";
                notSigned.AddOnlyProperties(o => o.Summary);
                notSigned.Remark = $"已匹配到OA请假申请,时间为:{oa.TimeStr:yyyy-MM-dd}至{oa.LateTime:yyyy-MM-dd}";
                notSigned.AddOnlyProperties(o => o.Remark);
                connHr.UpdateOnly(notSigned);
                success++;
            }
            TaskContext.Message = $"查询到{total}条未打卡记录,成功同步OA请假申请{success}条!";
            return true;
        }
    }
}
